#include "FBullCowGame.h"
#include <iostream>

FBullCowGame::FBullCowGame() { Reset(); } // Constructor calls for Reset() function.

// Getter functions.

int32 FBullCowGame::GetCurrentTry() const { return MyCurrentTry; } 
int32 FBullCowGame::GetHiddenWordLength() const { return HiddenWord.length(); }
bool FBullCowGame::GetIsGameWon() const { return bGameIsWon; }
int32 FBullCowGame::GetMaxTries() const {
	TMap<int32, int32> WordLengthToMaxTries{ {3,5}, {4,8}, {5,10}, {6,15}, {7, 20}, {8, 25} };
	return WordLengthToMaxTries[GetHiddenWordLength()];
}

// Resets the game by setting default values for the game.

void FBullCowGame::Reset() {
	HiddenWord = HiddenWordList[RandomNumberGen()];
	MyCurrentTry = 1;
	bGameIsWon = false;
	return;
}

// Checks users guess correctness.

EGuessStatus FBullCowGame::CheckGuessValidity(FString Guess) const {
	if (!IsIsogram(Guess)) {
		return EGuessStatus::Not_Isogram;
	}
	else if (!IsLowerCase(Guess)) {
		return EGuessStatus::Not_Lowercase;
	}
	else if (GetHiddenWordLength() != Guess.length()) {
		return EGuessStatus::Wrong_length;
	}
	else if (!IsOneWord(Guess)) {
		return EGuessStatus::Not_One_Word;
	}
	else {
		return EGuessStatus::OK;
	}
}

// Counts tries, bulls, cows and checks if game is won.

FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess) {
	
	MyCurrentTry++;
	
	FBullCowCount BullCowCount;
	int32 WordLength = GetHiddenWordLength();
	for (int32 MHWChar = 0; MHWChar < WordLength; MHWChar++) {
		for (int32 GChar = 0; GChar < WordLength; GChar++) {
			if (Guess[MHWChar] == HiddenWord[MHWChar]) {
				BullCowCount.Bulls++;
				break;
			}
			if(Guess[MHWChar] == HiddenWord[GChar]){
				BullCowCount.Cows++;
			}
		}
	}
	if (BullCowCount.Bulls == WordLength) {
		bGameIsWon = true;
	}
	else {
		bGameIsWon = false;
	}
	return BullCowCount;
}

// Checks if all characters are lowercase.

bool FBullCowGame::IsLowerCase(FString Guess) const {
	for (auto Letter : Guess) {
		if (Letter != tolower(Letter)) {
			return false;
		}
	}
	return true;
}

// Checks if word is an isogram.

bool FBullCowGame::IsIsogram(FString Guess) const {
	if (Guess.length() < 2) { return true; }
	TMap<char, bool> LetterSeen;
	for (auto Letter : Guess) {
		Letter = tolower(Letter);
		if (LetterSeen[Letter]) {
			return false;
		}
		else {
			LetterSeen[Letter] = true;
		}
	}
		return true;
}

// Checks if input is one word.

bool FBullCowGame::IsOneWord(FString Guess) const {
	for (char Letter : Guess) {
		if (Letter == char(32)) {
			return false;
		}
	}
	return true;
}

// Generates a random integer between the range of 0 and 17.

int32 FBullCowGame::RandomNumberGen() {
	srand(time(NULL));
	return rand() % 18;
}
