/* The game logic (no view code or direct user interaction).
The game is a word game based on Mastermind.
*/
#pragma once
#include <string>
#include <cstdlib>
#include <ctime>
#include <map>
#define TMap std::map

// Makes our code more Unreal like.

using FString = std::string;
using int32 = int;

// Stores small variables.

struct FBullCowCount {
	int32 Bulls = 0;
	int32 Cows = 0;
};

// Declares potential guess statuses.

enum class EGuessStatus {
	Invalid_Status,
	OK,
	Not_Isogram,
	Wrong_length,
	Not_Lowercase,
	Not_One_Word,
};



class FBullCowGame {
public:
	FBullCowGame(); // Constructor.

	// Declares functions to be used in FBullCowGame.cpp. 

	int32 GetMaxTries() const;
	int32 GetCurrentTry() const;
	int32 GetHiddenWordLength() const;
	int32 RandomNumberGen();
	EGuessStatus CheckGuessValidity(FString) const; 
	FBullCowCount SubmitValidGuess(FString);
	bool GetIsGameWon() const;
	void Reset();

private:
	// Declares variables to be used in FBullCowGame.cpp. 

	int32 MyCurrentTry;
	int32 MyMaxTries;
	
	// These must be isograms!

	FString HiddenWordList[18] = {"gum", "car", "ant", "weak", "wing", "true", "demon", "tower", "pearl", "spider", "satire", "parody", "dynamic", "conquer", "kingdom", "anorexic", "frosting", "graphics"};
	FString HiddenWord;
	bool bGameIsWon;
	bool IsLowerCase(FString) const;
	bool IsIsogram(FString) const;
	bool IsOneWord(FString) const;
};
