/* This is the console executable, that makes use of the BullCow class.
For game logic see the FBullCowGame class.
*/

#pragma once
#include <iostream>
#include <string>
#include "FBullCowGame.h"

// Function prototypes.

using FText = std::string;
using int32 = int;
void PrintIntro();
FText GetValidGuess();
void PlayGame();
bool AskToPlayAgain();
void PrintGameSummary(bool WinLose);

FBullCowGame BCGame; // Creates class object.

// First function that is called.

int main() {
	do {
		PlayGame();
	}
	while(AskToPlayAgain());
	return 0;
}

// Prints games introduction.

void PrintIntro() {
	std::cout << "Welcome to Bulls and Cows!" << std::endl;
	std::cout << "Can you guess the " << BCGame.GetHiddenWordLength() << " letter isogram I am thinking of?" << std::endl;
	std::cout << R"(
          .=     ,        =.
  _  _   /'/    )\,/,/(_   \ \
   `//-.|  (  ,\\)\//\)\/_  ) |
   //___\   `\\\/\\/\/\\///'  /
,-"~`-._ `"--'_   `"""`  _ \`'"~-,_
\       `-.  '_`.      .'_` \ ,-"~`/
 `.__.-'`/   (-\        /-) |-.__,'
   ||   |     \O)  /^\ (O/  |
   `\\  |         /   `\    /
     \\  \       /      `\ /
      `\\ `-.  /' .---.--.\
        `\\/`~(, '()      ('
         /(O) \\   _,.-.,_)
        //  \\ `\'`      /
       / |  ||   `""""~"`
     /'  |__||
           `o
		Art by Joan Stark
	)" << "\n";
	return;
}

// Initiates the game.

void PlayGame() {
	BCGame.Reset(); // Resets the game. 
	PrintIntro();
	std::cout << "Maximum tries " << BCGame.GetMaxTries() << ".\n" << std::endl;
	while( !BCGame.GetIsGameWon() && BCGame.GetCurrentTry() <= BCGame.GetMaxTries()) {
		FText Guess = GetValidGuess();
		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);
		std::cout << "Bulls = " << BullCowCount.Bulls;
		std::cout << " Cows = " << BullCowCount.Cows << std::endl;
		std::cout << std::endl;
	}
	PrintGameSummary(BCGame.GetIsGameWon());
	return;
}

// Checks the guess validity.

FText GetValidGuess() {
	EGuessStatus Status = EGuessStatus::Invalid_Status;
	FText Guess = "";
	do {
		int32 CurrentTry = BCGame.GetCurrentTry();
		std::cout << "Try " << CurrentTry << " of " << BCGame.GetMaxTries() << ".\nPlease enter your " << BCGame.GetHiddenWordLength() << " letter guess: ";
		std::getline(std::cin, Guess);
		Status = BCGame.CheckGuessValidity(Guess);
		switch (Status) {
		case EGuessStatus::Wrong_length:
			std::cout << "PLEASE ENTER A " << BCGame.GetHiddenWordLength() << " LETTER WORD.\n" << std::endl;
			break;
		case EGuessStatus::Not_Isogram:
			std::cout << "PLEASE ENTER AN ISOGRAM.\n" << std::endl;
			break;
		case EGuessStatus::Not_Lowercase:
			std::cout << "PLEASE ENTER WORD IN ALL LOWERCASE.\n" << std::endl;
			break;
		case EGuessStatus::Not_One_Word:
			std::cout << "PLEASE ENTER A SINGLE WORD AND AVOID USING SPACES.\n" << std::endl;
		default:
			break;
		}
	} while (Status != EGuessStatus::OK);
	return Guess;
}

// Asks if user wants to play again.

bool AskToPlayAgain() {
	std::cout << "Do you wish to play again(y/n)?" << std::endl;
	FText Response = "";
	std::getline(std::cin, Response);
	return (Response[0] == 'y' || Response[0] == 'Y');
}

// Prints games conclusion.

void PrintGameSummary(bool WinLose) {
	if (WinLose) {
		std::cout << "YOU WON, CONGRATULATIONS!" << std::endl;
	}
	else {
		std::cout << "YOU LOST, BETTER LUCK NEXT TIME!" << std::endl;
	}
}
